#include "String.h"

int String::memoryUsed;
int String::maxMemoryUsed;
int String::copiesMade;
int String::movesMade;

String::String() :
	len(0),str(nullptr) {
}

String::String(const char* otherStr) {
	len = 0;
	while(otherStr[len] != '\0')
		++len;
	
	str = allocMemory(len);

	for(int n = 0;n < len;n++)
		str[n] = otherStr[n];
}

String::String(const String& other) {
	copiesMade++;
	len = other.len;
	str = allocMemory(len);
	for(int n = 0;n < len;n++)
		str[n] = other.str[n];
}

String::String(String&& other) noexcept {
	movesMade++;
	len = other.len;
	other.len = 0;
	str = other.str;
    other.str = nullptr;
}

String::~String() {
	if(str != nullptr) {
		memoryUsed -= len;
		delete[] str;
		len = 0;
		str = nullptr;
	}
}

String& String::operator=(const String& other) {
	if(str != nullptr) {
		memoryUsed -= len;
		delete[] str;
	}
	copiesMade++;
	len = other.len;
	str = allocMemory(len);
	for(int n = 0;n < len;n++)
		str[n] = other.str[n];
	return *this;
}

String& String::operator=(String&& other) {
	if(str != nullptr) {
		memoryUsed -= len;
		delete[] str;
	}
	movesMade++;
	len = other.len;
	other.len = 0;
	str = other.str;
	other.str = nullptr;
	return *this;
}

int String::length() const {
	return len;
}

void String::initString() {
	memoryUsed = 0;
	maxMemoryUsed = 0;
	copiesMade = 0;
	movesMade = 0;
}

int String::memoryUsage() {
	return memoryUsed;
}

int String::maxMemoryUsage() {
	return maxMemoryUsed;
}

int String::copyCount() {
	return copiesMade;
}

int String::moveCount() {
	return movesMade;
}

char* String::allocMemory(int size) {
	memoryUsed += size;
	if(memoryUsed > maxMemoryUsed)
		maxMemoryUsed = memoryUsed;
	return new char[size];
}

bool operator==(const String& one,const String& two)
{
	if(one.length() != two.length())
		return false;
	int max = one.length();
	for(int n = 0;n < max;n++)
		if(one.str[n] != two.str[n])
			return false;
	return true;
}

std::ostream& operator<<(std::ostream& out,const String& str)
{
	for(int n = 0;n < str.len;n++)
		out << str.str[n];
	return out;
}
